@@ -30,12 +30,16 @@ extern unsigned short caml_win32_revision;
 #include "misc.h"
 #include "memory.h"
 
+#define Io_interrupted (-1)
+
 /* Read at most [n] bytes from file descriptor [fd] into buffer [buf].
    [flags] indicates whether [fd] is a socket
    (bit [CHANNEL_FLAG_FROM_SOCKET] is set in this case, see [io.h]).
    (This distinction matters for Win32, but not for Unix.)
    Return number of bytes read.
-   In case of error, raises [Sys_error] or [Sys_blocked_io]. */
+   In case of error, raises [Sys_error] or [Sys_blocked_io].
+   If interrupted by a signal and no bytes where read, returns
+   Io_interrupted without raising. */
 extern int caml_read_fd(int fd, int flags, void * buf, int n);
 
 /* Write at most [n] bytes from buffer [buf] onto file descriptor [fd].
@@ -43,7 +47,9 @@ extern int caml_read_fd(int fd, int flags, void * buf, int n);
    (bit [CHANNEL_FLAG_FROM_SOCKET] is set in this case, see [io.h]).
    (This distinction matters for Win32, but not for Unix.)
    Return number of bytes written.
-   In case of error, raises [Sys_error] or [Sys_blocked_io]. */
+   In case of error, raises [Sys_error] or [Sys_blocked_io].
+   If interrupted by a signal and no bytes were written, returns
+   Io_interrupted without raising. */
 extern int caml_write_fd(int fd, int flags, void * buf, int n);
 
 /* Decompose the given path into a list of directories, and add them
@@ -85,11 +91,6 @@ extern void * caml_globalsym(const char * name);
 /* Return an error message describing the most recent dynlink failure. */
 extern char * caml_dlerror(void);
 
-/* Add to [contents] the (short) names of the files contained in
-   the directory named [dirname].  No entries are added for [.] and [..].
-   Return 0 on success, -1 on error; set errno in the case of error. */
-extern int caml_read_directory(char_os * dirname, struct ext_table * contents);
-
 /* Recover executable name if possible (/proc/sef/exe under Linux,
    GetModuleFileName under Windows).  Return NULL on error,
    string allocated with [caml_stat_alloc] on success. */
@@ -117,11 +118,11 @@ extern wchar_t *caml_win32_getenv(wchar_t const *);
 
 /* Windows Unicode support */
 
-extern int win_multi_byte_to_wide_char(const char* s,
+CAMLextern int win_multi_byte_to_wide_char(const char* s,
                                        int slen,
                                        wchar_t *out,
                                        int outlen);
-extern int win_wide_char_to_multi_byte(const wchar_t* s,
+CAMLextern int win_wide_char_to_multi_byte(const wchar_t* s,
                                        int slen,
                                        char *out,
                                        int outlen);
@@ -134,7 +135,7 @@ extern int win_wide_char_to_multi_byte(const wchar_t* s,
    The returned string is allocated with [caml_stat_alloc], so it should be free
    using [caml_stat_free].
 */
-extern wchar_t* caml_stat_strdup_to_utf16(const char *s);
+CAMLextern wchar_t* caml_stat_strdup_to_utf16(const char *s);
 
 /* [caml_stat_strdup_of_utf16(s)] returns a NULL-terminated copy of [s],
    re-encoded in UTF-8 if [caml_windows_unicode_runtime_enabled] is non-zero or
@@ -143,15 +144,17 @@ extern wchar_t* caml_stat_strdup_to_utf16(const char *s);
    The returned string is allocated with [caml_stat_alloc], so it should be free
    using [caml_stat_free].
 */
-extern char* caml_stat_strdup_of_utf16(const wchar_t *s);
+CAMLextern char* caml_stat_strdup_of_utf16(const wchar_t *s);
 
 /* [caml_copy_string_of_utf16(s)] returns an OCaml string containing a copy of
    [s] re-encoded in UTF-8 if [caml_windows_unicode_runtime_enabled] is non-zero
    or in the current code page otherwise.
 */
-extern value caml_copy_string_of_utf16(const wchar_t *s);
+CAMLextern value caml_copy_string_of_utf16(const wchar_t *s);
 
-extern int caml_win32_isatty(int fd);
+CAMLextern int caml_win32_isatty(int fd);
+
+CAMLextern void caml_expand_command_line (int *, wchar_t ***);
 
 #endif /* _WIN32 */
 
